import java.util.ArrayList;

public class Loc {
	public final double x;
	public final double y;
	public final double z;

	public Loc() {
		this(0,0,0);
	}

	public Loc(int var1, int var2) {
		this(var1,0,var2);
	}

	public Loc(int var1, int var2, int var3) {
		this((double)var1,(double)var2,(double)var3);
	}

	public Loc(double var1, double var3) {
		this(var1,0.0D,var3);
	}

	public Loc(ry var1) {
		this(var1.v().a,var1.v().b,var1.v().c);
	}

	public Loc(Loc loc) {
		this(loc.x,loc.y,loc.z);
	}

	public Loc(double var1, double var3, double var5) {
		x = var1;
		y = var3;
		z = var5;
	}

	public int x() {
		return (int)x;
	}

	public int y() {
		return (int)y;
	}

	public int z() {
		return (int)z;
	}

	public Loc add(int var1, int var2, int var3) {
		return new Loc(x + (double)var1,y + (double)var2,z + (double)var3);
	}

	public Loc add(double var1, double var3, double var5) {
		return new Loc(x + var1,y + var3,z + var5);
	}

	public Loc add(Loc var1) {
		return new Loc(x + var1.x,y + var1.y,z + var1.z);
	}

	public Loc subtract(int var1, int var2, int var3) {
		return new Loc(x - (double)var1,y - (double)var2,z - (double)var3);
	}

	public Loc subtract(double var1, double var3, double var5) {
		return new Loc(x - var1,y - var3,z - var5);
	}

	public Loc subtract(Loc var1) {
		return new Loc(x - var1.x,y - var1.y,z - var1.z);
	}

	public Loc multiply(double var1, double var3, double var5) {
		return new Loc(x * var1,y * var3,z * var5);
	}

	public Loc getSide(int var1) {
		switch (var1) {
			case 0:
				return new Loc(x,y - 1,z);
			case 1:
				return new Loc(x,y + 1,z);
			case 2:
				return new Loc(x,y,z - 1);
			case 3:
				return new Loc(x,y,z + 1);
			case 4:
				return new Loc(x - 1,y,z);
			case 5:
				return new Loc(x + 1,y,z);
		}
		return new Loc(x,y,z);
	}

	public boolean equals(Object var1) {
		if (var1 instanceof Loc) {
			Loc var2 = (Loc)var1;
			return x == var2.x && y == var2.y && z == var2.z;
		} else return false;
	}

	public String toString() {
		return "(" + x + "," + y + "," + z + ")";
	}

	public int distSimple(Loc var1) {
		return (int)(Math.abs(x - var1.x) + Math.abs(y - var1.y) + Math.abs(z - var1.z));
	}

	public double distAdv(Loc var1) {
		return Math.sqrt(Math.pow(x - var1.x,2.0D) + Math.pow(y - var1.y,2.0D) + Math.pow(z - var1.z,2.0D));
	}

	public static Loc[] vecAdjacent() {
		Loc[] var0 = new Loc[]{new Loc(0,0,1),new Loc(0,0,-1),new Loc(0,1,0),new Loc(0,-1,0),new Loc(1,0,0),new Loc(-1,0,0)};
		return var0;
	}

	public Loc[] adjacent() {
		Loc[] var1 = vecAdjacent();
		for (int var2 = 0; var2 < var1.length; ++var2)
			var1[var2] = add(var1[var2]);
		return var1;
	}

	public static Loc[] vecAdjacent2D() {
		Loc[] var0 = new Loc[]{new Loc(0,1),new Loc(0,-1),new Loc(1,0),new Loc(-1,0)};
		return var0;
	}

	public Loc[] adjacent2D() {
		Loc[] var1 = vecAdjacent();
		for (int var2 = 0; var2 < var1.length; ++var2)
			var1[var2] = add(var1[var2]);
		return var1;
	}

	public static ArrayList<Loc> vecInRadius(int var0, boolean var1) {
		ArrayList<Loc> var2 = new ArrayList<Loc>();
		Loc var3 = new Loc();

		for (int var4 = -var0; var4 <= var0; ++var4) {
			for (int var5 = -var0; var5 <= var0; ++var5) {
				for (int var6 = -var0; var6 <= var0; ++var6) {
					Loc var7 = new Loc(var4,var6,var5);
					double var8 = var1 ? var3.distAdv(var7) : (double)var3.distSimple(var7);
					if (var8 <= (double)var0) var2.add(var7);
				}
			}
		}
		return var2;
	}

	public ArrayList<Loc> inRadius(int var1, boolean var2) {
		ArrayList<Loc> var3 = new ArrayList<Loc>();

		for (int var4 = -var1; var4 <= var1; ++var4) {
			for (int var5 = -var1; var5 <= var1; ++var5) {
				for (int var6 = -var1; var6 <= var1; ++var6) {
					Loc var7 = (new Loc(var4,var6,var5)).add(this);
					double var8 = var2 ? distAdv(var7) : (double)distSimple(var7);
					if (var8 <= (double)var1) var3.add(var7);
				}
			}
		}
		return var3;
	}

	public static ArrayList<Loc> vecInRadius2D(int var0, boolean var1) {
		ArrayList<Loc> var2 = new ArrayList<Loc>();
		Loc var3 = new Loc();

		for (int var4 = -var0; var4 <= var0; ++var4) {
			for (int var5 = -var0; var5 <= var0; ++var5) {
				Loc var6 = new Loc(var4,var5);
				double var7 = var1 ? var3.distAdv(var6) : (double)var3.distSimple(var6);
				if (var7 <= (double)var0) var2.add(var6);
			}
		}
		return var2;
	}

	public ArrayList<Loc> inRadius2D(int var1, boolean var2) {
		ArrayList<Loc> var3 = new ArrayList<Loc>();

		for (int var4 = -var1; var4 <= var1; ++var4) {
			for (int var5 = -var1; var5 <= var1; ++var5) {
				Loc var6 = (new Loc(var4,var5)).add(this);
				double var7 = var2 ? distAdv(var6) : (double)distSimple(var6);
				if (var7 <= (double)var1) var3.add(var6);
			}
		}
		return var3;
	}

	public int getBlock(kq var1) {
		return var1.a(x(),y(),z());
	}

	public Loc setBlockNotify(ry var1, int var2) {
		var1.g(x(),y(),z(),var2);
		return this;
	}

	public Loc setBlock(ry var1, int var2) {
		var1.d(x(),y(),z(),var2);
		return this;
	}

	public int getMeta(kq var1) {
		return var1.d(x(),y(),z());
	}

	public Loc setMeta(ry var1, int var2) {
		var1.c(x(),y(),z(),var2);
		return this;
	}

	public Loc setMetaNotify(ry var1, int var2) {
		var1.f(x(),y(),z(),var2);
		return this;
	}

	public Loc setBlockAndMeta(ry var1, int var2, int var3) {
		var1.b(x(),y(),z(),var2,var3);
		return this;
	}

	public Loc setBlockAndMetaNotify(ry var1, int var2, int var3) {
		var1.d(x(),y(),z(),var2,var3);
		return this;
	}

	public bq getTileEntity(kq var1) {
		return var1.b(x(),y(),z());
	}

	public Loc setTileEntity(ry var1, bq var2) {
		var1.a(x(),y(),z(),var2);
		return this;
	}

	public Loc removeTileEntity(ry var1) {
		var1.o(x(),y(),z());
		return this;
	}

	public int getLight(ry var1) {
		return var1.m(x(),y(),z());
	}

	public Loc notify(ry var1) {
		var1.j(x(),y(),z(),getBlock(var1));
		return this;
	}

	public Loc setSpawnPoint(ry var1) {
		var1.a(new dh(x(),y(),z()));
		return this;
	}
}